﻿using System.Collections.Generic;
using System.Threading;

namespace TestProject
{
    /// <summary>
    /// 고객 공급자
    /// </summary>
    public class CustomerProvider : IItemProvider<Customer>
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 카운트
        /// </summary>
        private readonly int count;

        /// <summary>
        /// 페치 지연 시간
        /// </summary>
        private readonly int fetchDelay;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomerProvider(count, fetchDelay)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="count">카운트</param>
        /// <param name="fetchDelay">페치 지연 시간</param>
        public CustomerProvider(int count, int fetchDelay)
        {
            this.count      = count;
            this.fetchDelay = fetchDelay;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 카운트 가져오기 - FetchCount()

        /// <summary>
        /// 카운트 가져오기
        /// </summary>
        /// <returns>카운트</returns>
        public int FetchCount()
        {
            Thread.Sleep(this.fetchDelay);

            return this.count; 
        }

        #endregion
        #region 범위 가져오기 - FetchRange(startIndex, count)

        /// <summary>
        /// 범위 가져오기
        /// </summary>
        /// <param name="startIndex">시작 인덱스</param>
        /// <param name="count">카운트</param>
        /// <returns>고객 리스트</returns>
        public IList<Customer> FetchRange(int startIndex, int count)
        {
            Thread.Sleep(this.fetchDelay);

            List<Customer> list = new List<Customer>();

            for(int i = startIndex; i < startIndex + count; i++)
            {
                Customer customer = new Customer { ID = i + 1, Name = "고객 " + (i + 1) };

                list.Add(customer);
            }

            return list;
        }

        #endregion
    }
}